"""Constants for the onboarding component."""
DOMAIN = 'onboarding'
STEP_USER = 'user'
STEP_CORE_CONFIG = 'core_config'
STEP_INTEGRATION = 'integration'

STEPS = [
    STEP_USER,
    STEP_CORE_CONFIG,
    STEP_INTEGRATION,
]

DEFAULT_AREAS = (
    'living_room',
    'kitchen',
    'bedroom',
)
