"""Automatically generated by hassfest.

To update, run python3 -m script.hassfest
"""


SSDP = {
    "device_type": {},
    "manufacturer": {
        "Belkin International Inc.": [
            "wemo"
        ],
        "Royal Philips Electronics": [
            "deconz",
            "hue"
        ]
    },
    "st": {
        "urn:schemas-denon-com:device:ACT-Denon:1": [
            "heos"
        ],
        "urn:schemas-upnp-org:device:ZonePlayer:1": [
            "sonos"
        ]
    }
}
