"""Constants for Life360 integration."""
DOMAIN = 'life360'

CONF_AUTHORIZATION = 'authorization'
CONF_CIRCLES = 'circles'
CONF_DRIVING_SPEED = 'driving_speed'
CONF_ERROR_THRESHOLD = 'error_threshold'
CONF_MAX_GPS_ACCURACY = 'max_gps_accuracy'
CONF_MAX_UPDATE_WAIT = 'max_update_wait'
CONF_MEMBERS = 'members'
CONF_SHOW_AS_STATE = 'show_as_state'
CONF_WARNING_THRESHOLD = 'warning_threshold'

SHOW_DRIVING = 'driving'
SHOW_MOVING = 'moving'
